#include <EEPROM.h>

//Press to Dim
//Christian Lysholm 2015
//A single button both turns on the light and dims it when held
//If light is off when button is held, transitions to off state (using external relays)

//Initialize Variables
 const int buttonPin = 4;
 const int ledPin = 0;
 const int buttonLedPin = 1;
 const int offPin = 2;
 
 const unsigned long holdLength = 1000; //minimum hold for 'long press'
 
 int ledState = 0; //state of LED. 0 -> off;
 int fadeStep = -3; //Speed of fade, sign indicates direction
 int fadeStep2 = 5; // Speed of button LED fade
 unsigned long buttonPressStart = 0; //time of button press
 int buttonState = 0; //state of the button . 0--> off
 int buttonPrevState = 0; //previous state of button
 int brightness = EEPROM.read(0); //initialize brightness from EEPROM cell 0
 int brightness2 = 5; //initialize the button ring LED nearly off
 
 void setup() {
  //Pin Setup
  pinMode(buttonPin, INPUT_PULLUP);
  pinMode(ledPin, OUTPUT);
  pinMode(buttonLedPin, OUTPUT);
  pinMode(offPin, OUTPUT);
  digitalWrite(offPin, 0);
  //Serial.begin(9600); //for debugging
 }

 void loop() {
  buttonState = !digitalRead(buttonPin); //low -->button is pressed
  //button is pressed and was not pressed before 
  //NEW PRESS BEGINING
  if ((buttonState == 1)&& (buttonPrevState == 0)) { 
    buttonPressStart = millis();
  }
  //button is not pressed and was pressed before
  //BUTTON PRESS HAS ENDED
  if ((buttonState == 0)&& (buttonPrevState == 1)) { 
    //Check to see if the button press was short or long
    if (buttonPressStart + holdLength > millis()){
      //short press finished
      ledState = !ledState;
    }
     if ((buttonPressStart + holdLength < millis()) && (ledState == 0) ){
      //Long press finished and LED is already off
      EEPROM.write(0,brightness);
      delay(5);
      digitalWrite(offPin,1);
    }
    else { //long press finished and LED was on
      //toggle fade direction
      fadeStep = -fadeStep;
    }
  }
  //button is pressed and was pressed before
  //BUTTON PRESS IS SUSTAINING
  if ((buttonState == 1)&&(buttonPrevState == 1)&&(ledState == 1)) {
    //check to see if the button press is short or long
    if ( buttonPressStart + holdLength < millis()){
      //Sustaining Long press, begin fade
      brightness = brightness + fadeStep;      
      if (brightness <= 0 || brightness >= 255) {
      fadeStep = -fadeStep;
        if(brightness <= 0){brightness = 1;}
        if(brightness >= 255){brightness = 254;} 
       } 
     }
  }
  //BUTTON PRESS IS SUSTAINING, LED IS OFF, AND LONG PRESS
  if ((buttonState == 1)&&(buttonPrevState == 1)&&(ledState == 0)&&(buttonPressStart + holdLength < millis())) {
    analogWrite(buttonLedPin, 0);
    delay(300);
    fadeStep2 = 0;
    brightness2 = 254;
  }
 if (brightness2 < 5 || brightness2 >= 200){
   fadeStep2 = -fadeStep2; 
  }

 brightness2 = brightness2 + fadeStep2;
  
 if(ledState == 0){analogWrite(ledPin,0);}
 else{analogWrite(ledPin,brightness);}
 buttonPrevState = buttonState;
 analogWrite(buttonLedPin, brightness2);
 //Serial.println(brightness); //for debugging
 delay(30);
 }

